import axios, { AxiosError } from 'axios';

class FranceTravailService {
  private clientId: string;
  private clientSecret: string;
  private tokenUrl: string;
  private apiUrl: string;
  private scope: string;
  private accessToken: string | null = null;
  private tokenExpiration: number | null = null;
  private realm: string = '/partenaire';

  constructor() {
    this.clientId = process.env.FRANCE_TRAVAIL_CLIENT_ID || '';
    this.clientSecret = process.env.FRANCE_TRAVAIL_CLIENT_SECRET || '';
    this.tokenUrl = process.env.FRANCE_TRAVAIL_TOKEN_URL || '';
    this.apiUrl = process.env.FRANCE_TRAVAIL_API_URL || '';
    this.scope = process.env.FRANCE_TRAVAIL_SCOPE || '';
  }

  private async authenticate(): Promise<void> {
    try {
      const response = await axios.post(this.tokenUrl, null, {
        params: {
          realm: this.realm,
          grant_type: 'client_credentials',
          client_id: this.clientId,
          client_secret: this.clientSecret,
          scope: this.scope,
        },
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded',
        },
      });

      this.accessToken = response.data.access_token;
      this.tokenExpiration = Date.now() + response.data.expires_in * 1000;
    } catch (error) {
      const axiosError = error as AxiosError;
      console.error('Authentication failed:', axiosError.response?.data || axiosError.message);
      throw new Error('Failed to authenticate with France Travail API');
    }
  }

  private async ensureValidToken(): Promise<void> {
    if (!this.accessToken || (this.tokenExpiration && Date.now() >= this.tokenExpiration)) {
      await this.authenticate();
    }
  }

  public async getJobOffers(params?: any): Promise<any> {
    await this.ensureValidToken();

    try {
      const response = await axios.get(this.apiUrl, {
        headers: {
          Authorization: `Bearer ${this.accessToken}`,
        },
        params: {
          ...params,
          range: params?.range || '0-9', // Default range for pagination
        },
      });

      return response.data;
    } catch (error) {
      const axiosError = error as AxiosError;
      console.error('Failed to fetch job offers:', axiosError.response?.data || axiosError.message);
      throw new Error('Failed to fetch job offers from France Travail API');
    }
  }
}

export default new FranceTravailService();
